<?php

namespace Drupal\drupalfit\Controller;

use Drupal\Core\Config\Config;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\drupalfit\Service\Drupalfit;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\system\SystemManager;

/**
 * Controller for the Drupalfit API.
 */
class DrupalfitApi extends ControllerBase {
  use StringTranslationTrait;

  /**
   * The configuration.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The system manager service.
   *
   * @var \Drupal\system\SystemManager
   */
  protected $systemManager;

  /**
   * The status report service.
   *
   * @var \Drupal\drupalfit\Service\Drupalfit
   */
  protected $drupalfit;

  /**
   * Constructs a DrupalfitApi object.
   *
   * @param \Drupal\Core\Config\Config $config
   *   The configuration.
   * @param \Drupal\system\SystemManager $systemManager
   *   An injected system.manager service.
   * @param \Drupal\drupalfit\Service\Drupalfit $drupalfit
   *   The drupalfit service.
   */
  public function __construct(Config $config, SystemManager $systemManager, Drupalfit $drupalfit) {
    $this->config = $config;
    $this->system_manager = $systemManager;
    $this->drupalfit = $drupalfit;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')->get('drupalfit.settings'),
      $container->get('system.manager'),
      $container->get('drupalfit.drupalfit_service')
    );
  }

  /**
   * Returns data as JSON.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response with status report data.
   */
  public function getData(Request $request) {
    // Get authorization api key.
    $drupalfit_api_key = $this->config->get('api_key');

    // Check for the API key in the request headers.
    $authorization_key = base64_decode($request->headers->get('Authorization'));
    // Validate the API key.
    if ($authorization_key && $authorization_key == $drupalfit_api_key) {
      // Get the status report information.
      $status_reports = $this->system_manager->listRequirements();
      // Get counter details.
      $counters_details = $this->drupalfit->getCountersDetails($status_reports);
      // Get general info.
      $general_info = $this->drupalfit->getGeneralInfo($status_reports);
      // Get group info.
      $group_info = $this->drupalfit->getGroupRequirements($status_reports);

      $response_data['status'] = 'success';
      $response_data['#counters'] = $counters_details['#counters'];
      $response_data['#general_info'] = $general_info['#general_info'];
      $response_data['#checked'] = $group_info['#grouped_requirements'];
      $response_code = 200;
    }
    else {
      // Return an unauthorized response if the key is invalid.
      $response_data['status'] = $this->t('error');
      $response_data['message'] = $this->t('Unauthorized');
      $response_code = 403;
    }

    // Return the data as a JSON response.
    $response = new JsonResponse($response_data, $response_code);

    // Or specify a specific origin.
    $response->headers->set('Access-Control-Allow-Origin', $this->config->get('base_url'));
    $response->headers->set('Access-Control-Allow-Methods', 'GET');
    $response->headers->set('Access-Control-Allow-Headers', 'Content-Type, Authorization, X-Requested-With');
    $response->headers->set('Access-Control-Allow-Credentials', 'true');

    return $response;

  }

}
