<?php

namespace Drupal\drupalfit\Service;

use Drupal\Core\StringTranslation\PluralTranslatableMarkup;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Service that return status page report data.
 */
class Drupalfit {
  use StringTranslationTrait;

  /**
   * Get counters details.
   */
  public function getCountersDetails($requirements) {
    // Count number of items with different severity for summary.
    $counters = [
      'error' => [
        'amount' => 0,
        'text' => $this->t('Error'),
        'text_plural' => $this->t('Errors'),
      ],
      'warning' => [
        'amount' => 0,
        'text' => $this->t('Warning'),
        'text_plural' => $this->t('Warnings'),
      ],
      'checked' => [
        'amount' => 0,
        'text' => $this->t('Checked', [], ['context' => 'Examined']),
        'text_plural' => $this->t('Checked', [], ['context' => 'Examined']),
      ],
    ];

    $severities = $this->getSeverities();
    foreach ($requirements as $key => &$requirement) {
      $severity = $severities[REQUIREMENT_INFO];
      if (isset($requirement['severity'])) {
        $severity = $severities[(int) $requirement['severity']];
      }
      elseif (defined('MAINTENANCE_MODE') && MAINTENANCE_MODE == 'install') {
        $severity = $severities[REQUIREMENT_OK];
      }

      if (isset($counters[$severity['status']])) {
        $counters[$severity['status']]['amount']++;
      }
    }

    foreach ($counters as $key => $counter) {
      if ($counter['amount'] === 0) {
        continue;
      }

      $text = new PluralTranslatableMarkup($counter['amount'], $counter['text'], $counter['text_plural']);

      $response['#counters'][$key] = [
        '#amount' => $counter['amount'],
        '#text' => $text->__toString(),
        '#severity' => $key,
      ];
    }

    return $response;
  }

  /**
   * Get gerneral info.
   */
  public function getGeneralInfo($requirements) {
    $element['#general_info'] = [];
    // Loop through requirements and pull out items.
    foreach ($requirements as $key => $requirement) {
      switch ($key) {
        case 'cron':
          $element['#general_info']['#' . $key]['title'] = $requirement['title']->__toString();
          $element['#general_info']['#' . $key]['value']['label'] = $requirement['value']->__toString();
          $element['#general_info']['#' . $key]['value']['time'] = $requirement['value']->getArguments()['@time'];
          break;

        case 'drupal':
          $element['#general_info']['#' . $key]['title'] = $requirement['title']->__toString();
          $element['#general_info']['#' . $key]['value'] = $requirement['value'];
          break;

        case 'webserver':
          $element['#general_info']['#' . $key]['title'] = $requirement['title']->__toString();
          $element['#general_info']['#' . $key]['value'] = $requirement['value'];
          break;

        case 'database_system':
          $element['#general_info']['#' . $key]['title'] = $requirement['title']->__toString();
          $element['#general_info']['#' . $key]['value'] = $requirement['value']->__toString();
          break;

        case 'database_system_version':
          $element['#general_info']['#' . $key]['title'] = $requirement['title']->__toString();
          $element['#general_info']['#' . $key]['value'] = $requirement['value'];
          break;

        case 'php':
          $element['#general_info']['#' . $key]['title'] = $requirement['title']->__toString();
          $element['#general_info']['#' . $key]['value'] = $requirement['value']->__toString();
          if ($requirement['description'] && gettype($requirement['description']) == 'object') {
            $element['#general_info']['#' . $key]['description'] = $requirement['description']->__toString();
          }
          else {
            $element['#general_info']['#' . $key]['description'] = $requirement['description'];
          }
          break;

        case 'php_memory_limit':
          $element['#general_info']['#' . $key]['title'] = $requirement['title']->__toString();
          $element['#general_info']['#' . $key]['value'] = $requirement['value'];
          break;
      }
    }

    return $element;
  }

  /**
   * Get group requirements.
   */
  public function getGroupRequirements($requirements) {
    $severities = $this->getSeverities();
    $grouped_requirements = [];
    foreach ($requirements as $key => $requirement) {
      $severity = $severities[REQUIREMENT_INFO];
      if (isset($requirement['severity'])) {
        $requirement_severity = (int) $requirement['severity'] === REQUIREMENT_OK ? REQUIREMENT_INFO : (int) $requirement['severity'];
        $severity = $severities[$requirement_severity];
      }
      elseif (defined('MAINTENANCE_MODE') && MAINTENANCE_MODE == 'install') {
        $severity = $severities[REQUIREMENT_OK];
      }
      $grouped_requirements[$severity['status']]['title'] = $severity['title']->__toString();
      $grouped_requirements[$severity['status']]['type'] = $severity['status'];
      if (array_key_exists('title', $requirement)) {
        if (gettype($requirement['title']) == 'object') {
          $grouped_requirements[$severity['status']]['items'][$key]['title'] = $requirement['title']->__toString();
        }
        else {
          $grouped_requirements[$severity['status']]['items'][$key]['title'] = $requirement['title'];
        }
      }
      if (array_key_exists('value', $requirement)) {
        if (gettype($requirement['value']) == 'object') {
          $grouped_requirements[$severity['status']]['items'][$key]['value'] = $requirement['value']->__toString();
        }
        else {
          $grouped_requirements[$severity['status']]['items'][$key]['value'] = $requirement['value'];
        }
      }
      if (array_key_exists('description', $requirement)) {
        if (gettype($requirement['description']) == 'object') {
          $grouped_requirements[$severity['status']]['items'][$key]['description'] = $requirement['description']->__toString();
        }
        else {
          if ($requirement['description'] && is_array($requirement['description'])) {
            if (array_key_exists('#markup', $requirement['description'])) {
              if (gettype($requirement['description']['#markup']) == 'object') {
                $grouped_requirements[$severity['status']]['items'][$key]['description'] = $requirement['description']['#markup']->__toString();
              }
              else {
                $grouped_requirements[$severity['status']]['items'][$key]['description'] = $requirement['description']['#markup'];
              }
            }
            else {
              foreach ($requirement['description'] as $desc_key => $description) {
                if (is_array($description)) {
                  foreach ($description as $subdesc_key => $sub_description) {
                    if (is_array($sub_description)) {
                      if (array_key_exists('#markup', $sub_description)) {
                        if (gettype($sub_description['#markup']) == 'object') {
                          $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key][$subdesc_key]['#markup']['label'] = $sub_description['#markup']->__toString();
                        }
                        else {
                          $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key][$subdesc_key]['#markup']['label'] = $sub_description['#markup'];
                        }
                        if ($sub_description['#markup']->getArguments()) {
                          $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key][$subdesc_key]['#markup']['cron_handbook'] = $sub_description['#markup']->getArguments()[':cron-handbook'];
                        }
                      }
                      else {
                        $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key][$subdesc_key]['#type'] = $sub_description['#type'];
                        if (gettype($sub_description['#title']) == 'object') {
                          $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key][$subdesc_key]['#title'] = $sub_description['#title']->__toString();
                        }
                        else {
                          $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key][$subdesc_key]['#title'] = $sub_description['#title'];
                        }
                        $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key][$subdesc_key]['#url'] = $sub_description['#url']->getRouteName();
                      }
                    }
                    else {
                      if (gettype($sub_description) == 'object') {
                        $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key][$subdesc_key]['label'] = $sub_description->__toString();
                      }
                      else {
                        $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key][$subdesc_key]['label'] = $sub_description;
                      }
                    }
                  }
                }
                else {
                  if (gettype($description) == 'object') {
                    $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key]['label'] = $description->__toString();
                  }
                  else {
                    $grouped_requirements[$severity['status']]['items'][$key]['description'][$desc_key]['label'] = $description;
                  }
                }
              }
            }
          }
          else {
            $grouped_requirements[$severity['status']]['items'][$key]['description'] = $requirement['description'];
          }
        }
      }
    }
    $priorities = [
      'error',
      'warning',
      'checked',
      'ok',
    ];
    // Order the grouped requirements by a set order.
    $order = array_flip($priorities);
    uksort($grouped_requirements, function ($a, $b) use ($order) {
      return $order[$a] <=> $order[$b];
    });

    $element['#grouped_requirements'] = $grouped_requirements;

    return $element;
  }

  /**
   * Gets the severities.
   *
   * @return array
   *   Serverities definition array
   */
  public function getSeverities() {
    return [
      REQUIREMENT_INFO => [
        'title' => $this->t('Checked', [], ['context' => 'Examined']),
        'status' => 'checked',
      ],
      REQUIREMENT_OK => [
        'title' => $this->t('OK'),
        'status' => 'ok',
      ],
      REQUIREMENT_WARNING => [
        'title' => $this->t('Warnings found'),
        'status' => 'warning',
      ],
      REQUIREMENT_ERROR => [
        'title' => $this->t('Errors found'),
        'status' => 'error',
      ],
    ];
  }

}
